/*
 *	$Id: long_line.c 2 2008-08-25 22:47:08Z rh $
 *
 * malloc(), free(), and realloc() demonstration
 */
#include <stdlib.h>
#include <stdio.h>

/*
 * a function that is able to read long lines on stdin
 * using malloc().  The line returned must be freed by
 * the receiver.
 */
char *get_long_line(FILE *in)
{
	int c = 0;
	unsigned long i = 0;
	size_t line_size = 80;			// initial line size is 80
	char *line = malloc(line_size);		// allocate memory for a line

	while (line != NULL && c != EOF)
	{
		while (i < line_size - 1)
		{
			if ((c = getc(in)) == EOF)
			{
				if (ferror(in))
				{
					free(line);
					return NULL;
				}
				break;
			}
			line[i++] = c;
			if (c == '\n' || c == '\0')
			{
				c = EOF;	// end of line was reached
				break;
			}
		}
		if (i >= line_size - 1)
		{
			line_size *= 2;		// double the line size
			line = realloc(line, line_size);
		}
	}
	if (line != NULL)
		line[i] = '\0';			// terminate string

	return line;
}


int main(int argc, char *argv[])
{
	printf("Input a long line: ");
	fflush(stdout);

	char *line = get_long_line(stdin);

	if (line == NULL)
	{
		perror("Cannot get line");
		return EXIT_FAILURE;
	}

	printf("Here is the line : %s", line);

	free(line);				// release memory!

	return EXIT_SUCCESS;
}
